#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>

//Blink-Frequenz in Hz
#define BLINK_FREQ 1

//90*60 ~ 1,5h
#define TIME_RESET 90 *60 * BLINK_FREQ
//10*60 ~ 10min
#define TIME_WARNING 10 * 60 * BLINK_FREQ
#define POFF_REPETITION 10
#define LED_WARNING (1<<PB4)

//Ab hier bitte nichts mehr verndern, wenn man nicht wei, was man macht

#if TIME_RESET > 65535
	#warning Reset-Time is too high, please reduce either TIME_RESET or BLINK_FREQ!
#endif

#define TIMER_POSTSCALER (F_CPU / 1024 / 256 / BLINK_FREQ)
#define TIMER_ACTIVE 255
#define TIMER_INACTIVE 0

#define LED_IR (1<<PB0)
#define IR_FREQ 38000
#define freq_to_timerval(x) ((F_CPU / x - 1)/ 2)

#define CMD_SPACE 74

#define START_H 3479
#define START_L 1833

#define BASETIME 450
#define CORR_HI -55
#define CORR_LO 54

//Gibt ein mit 38kHz moduliertes Signal am OCR0A (PB0) aus
void SendSignal(uint8_t high) {
	if(high == 1) {
		OCR0A = freq_to_timerval(IR_FREQ);
		TCCR0A = (1<<COM0A0) | (1<<WGM01);
	    TCCR0B = (1<<CS00);
	} else {
		TCCR0A = 0;
		TCCR0B = 0;
	}
}

//Startbedingung senden
void SendStart() {
	SendSignal(1);
	_delay_us(START_H);
	SendSignal(0);
	_delay_us(START_L);
}

//Ein Bit senden
void SendBit(uint8_t value) {
	SendSignal(1);
	_delay_us(BASETIME + CORR_HI);
	SendSignal(0);
	if(value == 0) {
		_delay_us(BASETIME + CORR_LO);
	} else {
		_delay_us(3*BASETIME + CORR_LO);
		
	}
}

//Ein Byte senden
void SendByte(uint8_t value) {
	uint8_t n = 8;
	while(n--) {
		SendBit(value & 128);
		value <<= 1;
	}
}
		
//Der Befehl fr's abschalten
inline static void PowerOff() {
	SendStart();
	SendByte(0x40);
	SendByte(0x04);
	SendByte(0x01);
	SendByte(0x00);
	SendByte(0xBC);
	SendByte(0xBD);
	SendBit(1); //STOP
	_delay_ms(CMD_SPACE);
}




volatile uint8_t mycount = 0;
volatile uint16_t time_of_death = TIME_RESET;
volatile uint8_t turn_off = 0;

void SetTimer(uint8_t active) {
	//Normalbetrieb:
	TCCR0A = (0<<COM0A1) | (0<<COM0A0) | (0<<COM0B1) | (0<<COM0B0) | (0<<WGM01) | (0<<WGM00);
	TCCR0B = (0<<FOC0A) | (0<<FOC0B) | (0<<WGM02) | (1<<CS02) | (0<<CS01) | (1<<CS00); //clk/1024
	TIMSK0 = (0<<OCIE0B) | (0<<OCIE0A) | ((1<<TOIE0) & active);
}

ISR(SIG_INTERRUPT0) {
	PORTB |= (1<<PB4); //Warn-LED blinken lassen, wenn etwas empfangen wurde
	time_of_death = TIME_RESET;
	SetTimer(TIMER_ACTIVE);
	/* Wieder abschalten, Nebeneffekt: Wenn der Timer-Of-Death abgebrochen wurde,
	 * wenn die LED gerade geleuchtet hat, wird sie ebenso abgeschalten
	 */
	PORTB &= ~LED_WARNING; 
}

ISR(SIG_OVERFLOW0) {
	mycount++;
	if(mycount > TIMER_POSTSCALER) {
		
		if(time_of_death <= TIME_WARNING) {
			PORTB ^= LED_WARNING; //Wenn die Warn-Zeit erreicht wurde, LED blinken lassen
		}
		if(time_of_death > 0) {
			time_of_death--;
		} else {
			PORTB &= ~LED_WARNING; //Warn-LED abschalten und das Flag zum abschalten setzen
			turn_off = 1;
		}
		mycount = 0;
	}
}

int main() {
	DDRB |= LED_IR; //IR-LED
	DDRB |= LED_WARNING; //Warn-LED
	DDRB &= ~(1<<PB1); //IR-Empfnger

	MCUCR &= ~((0<<ISC01) | (0<<ISC00)); //low level creates request
	GIMSK |= (1<<INT0);


	SetTimer(TIMER_ACTIVE);
	sei(); //interrupts aktivieren

	

	while(1) {
		if(turn_off == 1) {
			uint8_t cnt = POFF_REPETITION;

			GIMSK &= ~(1<<INT0); //IR-Empfnger blind machen

			SetTimer(TIMER_INACTIVE); //Timer deaktivieren
			while(cnt--) {
				PowerOff(); //Fernseher abschalten
			}

			GIMSK |= (1<<INT0); //IR-Empfnger wieder aktivieren
			turn_off = 0; //Flag zurcksetzen
			SetTimer(TIMER_INACTIVE); //Timer vorsichtshalber nochmal deaktivieren
		}

	} 

	return 0;
}
